#/*
# * +--------------------------------------------------------------------------+
# * | Copyright 1999, 2002 Rational Software Corporation.  All Rights Reserved.|
# * | This software contains proprietary and confidential information of       |
# * | Rational and its suppliers.  Use, disclosure or reproduction is          |
# * | prohibited without the prior express written consent of Rational.        |
# * +--------------------------------------------------------------------------+
# */
################################################################################
#
# updateMSPAggregate.pl
#
# This transformation calculates the current day's baseline task complete, current
# task complete and actual task complete counts and puts the results in the
# MSPAggregate_measure table.
#
################################################################################

use PjCLib;
use Win32::OLE;
require CQPerlExt;

use constant CURRENCY_SIGN => "\$";
use constant PERCENT_SIGN => "\%";
use constant CURRENCY_DEMARCATOR => "\,";
use constant LEFT_PAREN => "(";
use constant RIGHT_PAREN => ")";
use constant MINUS => "-";
use constant HOUR => " hr";
use constant HOURS => " hrs";
use constant DAY => " day";
use constant DAYS => " days";
use constant NULL_STRING => "";
use constant DBID => "DBID";
use constant YES => "yes";
use constant NO => "no";
use constant BOOL_TRUE => "True";
use constant BOOL_FALSE => "False";


################
# main
################

# global variables
$currentDate;
$currentTime;
getCurrentDateTime();

print "Start Time = $currentDate $currentTime \n";

$session = logon();


# If you wish to perform the calculation for a certain day, uncomment the
# following line and set it to a day with SQL date format;
#$currentDate = "1999-05-07";


$baselineFinish = "";
$actualFinish = "";
$collInstanceLogID = "";
$finish = "";

myMain();

getCurrentDateTime();
print "End Time = $currentDate $currentTime \n";

exit;


sub myMain {

    my $DBID;
    my $costStr;
    my $workStr;
    print "Updating MS Project aggregate table.  Please wait...\n";

    PjCLib->initializePjCLibrary($session, PjCLib->BOOLEAN_TRUE);

    initialize();


    updateAggregateMeasures();
}

sub initialize {

    # Get physical table names.  These names will be used in the query definitions.
    $PROJECT_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("MSP_Task_Measure");
    $PROJECT_AGGREGAGE_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("MSPAggregate_measure");

    $COLLECTIONDATE_TABLE = PjCLib->getPhysicalTableDbName("collectiondate");
    $CALENDARDATE_TABLE = PjCLib->getPhysicalTableDbName("calendardate");
    $PROJECTDATE_TABLE = PjCLib->getPhysicalTableDbName("ProjectDate");
    $ACTUALSTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("ActualStart");
    $ACTUALFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("ActualFinish");
    $BASELINESTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("BaselineStart");
    $BASELINEFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("BaselineFinish");
    $CURPLANSTARTDATE_TABLE = PjCLib->getPhysicalTableDbName("curplanstart");
    $CURPLANFINISHDATE_TABLE = PjCLib->getPhysicalTableDbName("curplanfinish");
    $MSP_MILESTONE_DIM = PjCLib->getPhysicalTableDbName("msp_milestone_dim");

    # Get physical field names for the task table.  These names will be used in the query definitions.
    $TASK_NAME = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "taskname_fk");
    $TASK_ID = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "taskid_fk");
    $MEASURE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Number of Tasks");
    $ACTUAL_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualCost");
    $ACTUAL_DURATION = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualDuration");
    $ACTUAL_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "actualfinish_fk");
    $ACTUAL_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "actualstart_fk");
    $ACTUAL_WORK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPActualWork");
    $ACWP = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "ACWP");
    $BASELINE_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineCost");
    $BASELINE_DURATION = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineDuration");
    $BASELINE_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "baselinefinish_fk");
    $BASELINE_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "baselinestart_fk");
    $CURRENT_PLAN_START_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "curplanstart_fk");
    $CURRENT_PLAN_FINISH_DATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "curplanfinish_fk");
    $BASELINE_WORK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPBaselineWork");
    $BCWP = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "BCWP");
    $BCWS = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "BCWS");
    $COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Cost_1");
    $COST_VARIANCE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "CostVariance");
    $FIXED_COST = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "FixedCost_1");
    $PERCENT_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPPercentComplete");
    $PERCENT_WORK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPPercentWorkComplete");
    $TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "MSPTaskComplete");
    $ONTIME_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "ontime_fk");
    $MILESTONE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "milestone_fk");
    $COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "collectiondate_fk");
    $COLLECTION_INSTANCE_ID = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "coll_instance_log_id");

    # Get physical field names for the aggregate table.  These names will be used in the query definitions.
    $MSP_AGGR_BP_TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: BP Task Complete");
    $MSP_AGGR_CP_TASK_COMPLETE = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: CP Task Complete");
    $MSP_AGGR_TASK_COMPLETE  = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "MSP Aggr: Task Complete");
    $MSP_AGGR_COLLECTIONDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "collectiondate_fk");
    $MSP_AGGR_CALENDARDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "calendardate_fk");
    $MSP_AGGR_COLLECTION_INSTANCE_ID = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "coll_instance_log_id");
    $MSP_AGGR_PROJECTDATE_FK = PjCLib->getPhysicalFieldDbName("MSPAggregate_measure", "projectdate_fk");

    # These fields currently not used.
    #$TRANSFORMED = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Transformed");
    #$MILESTONE = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "Milestone");
    #$MSP_AGGR_MILESTONE_FK = PjCLib->getPhysicalFieldDbName("MSP_Task_Measure", "milestone_fk");

}


sub replace {
    my ($text, $toReplace, $replaceWith,) = @_;

    #print "replace $text, $toReplace, $replaceWith\n";
    # replace all occurances

    my $notDone = 1;
    while($notDone) {
        my $index = index($text, $toReplace);
        if ($index >= 0) {
            substr($text, $index, length($toReplace), $replaceWith);
        } 
        else {
            $notDone = 0;
        }
    }
    #print "transformed to $text\n";
    return $text;
}

sub updateAggregateMeasures {

    my $minBPDate = getMinOrMaxDate($BASELINEFINISHDATE_TABLE, $BASELINE_FINISH_DATE_FK, "MIN");
    my $maxBPDate = getMinOrMaxDate($BASELINEFINISHDATE_TABLE, $BASELINE_FINISH_DATE_FK, "MAX");
    my $minCPDate = getMinOrMaxDate($CURPLANFINISHDATE_TABLE, $CURRENT_PLAN_FINISH_DATE_FK, "MIN");
    my $maxCPDate = getMinOrMaxDate($CURPLANFINISHDATE_TABLE, $CURRENT_PLAN_FINISH_DATE_FK, "MAX");
    my $minActualDate = getMinOrMaxDate($ACTUALFINISHDATE_TABLE, $ACTUAL_FINISH_DATE_FK, "MIN");
    my $maxActualDate     = getMinOrMaxDate($ACTUALFINISHDATE_TABLE, $ACTUAL_FINISH_DATE_FK, "MAX");
    printf "minBPDate     = $minBPDate;     maxBPDate     = $maxBPDate\n";
    printf "minCPDate     = $minCPDate;     maxCPDate     = $maxCPDate\n";
    printf "minActualDate = $minActualDate; maxActualDate = $maxActualDate\n";

    #
    # Return a result set containing the rows of measures that have an baselinefinish date
    #

    #print "updateAggregateMeasures():  get baseline finish dates...\n";
    $queryStr = "SELECT " . $PROJECTDATE_TABLE . "_name " .
                " FROM " . $PROJECTDATE_TABLE .
                " WHERE " . $PROJECTDATE_TABLE . "_name >= '$minBPDate' AND " .
                $PROJECTDATE_TABLE . "_name <= '$maxBPDate'";
    #print "Excute query $queryStr...\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    my $count = 0;
    print "Updating baseline task complete counts...\n";
    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $dateValue = $resultSet->GetColumnValue(1);
       #print "   dateValue = $dateValue\n";
       my $baselineFinishDBID = getDBID ($BASELINEFINISHDATE_TABLE,
                                         $BASELINEFINISHDATE_TABLE . "_name",
                                         $dateValue);
       my $numBPTasks = getAggregateValue($MEASURE,
                                          $BASELINE_FINISH_DATE_FK,
                                          $baselineFinishDBID,
                                          $dateValue);

       #printf "numBPTasks = $numBPTasks\n";

       updateAggregateTable($dateValue,
                            $MSP_AGGR_BP_TASK_COMPLETE,
                            $numBPTasks);
       
    }

    printf "Updated $count rows with baseline task complete.\n";

    #
    # Return a result set containing the rows of measures that have an current plan finish date
    #

    #print "updateAggregateMeasures():  get baseline finish dates...\n";
    $queryStr = "SELECT " . $PROJECTDATE_TABLE . "_name " .
                " FROM " . $PROJECTDATE_TABLE .
                " WHERE " . $PROJECTDATE_TABLE . "_name >= '$minCPDate' AND " .
                $PROJECTDATE_TABLE . "_name <= '$maxCPDate'";
    #print "Excute query $queryStr...\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    my $count = 0;
    print "Updating current task complete counts...\n";
    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $dateValue = $resultSet->GetColumnValue(1);
       #print "   dateValue = $dateValue\n";
       my $curPlanFinishDBID = getDBID ($CURPLANFINISHDATE_TABLE,
                                         $CURPLANFINISHDATE_TABLE . "_name",
                                         $dateValue);
       my $numCPTasks = getAggregateValue($MEASURE,
                                          $CURRENT_PLAN_FINISH_DATE_FK,
                                          $curPlanFinishDBID,
                                          $dateValue);


       #printf "numCPTasks = $numCPTasks\n";

       updateAggregateTable($dateValue,
                            MSP_AGGR_CP_TASK_COMPLETE,
                            $numCPTasks);
       
    }
    printf "Updated $count rows with current task complete.\n";


    #
    # Return a result set containing the rows of measures that have an actual finish date
    #

    #print "updateAggregateMeasures():  get baseline finish dates...\n";
    $queryStr = "SELECT " . $PROJECTDATE_TABLE . "_name " .
                " FROM " . $PROJECTDATE_TABLE .
                " WHERE " . $PROJECTDATE_TABLE . "_name >= '$minActualDate' AND " .
                $PROJECTDATE_TABLE . "_name <= '$maxActualDate'";
    #print "Excute query $queryStr...\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    my $count = 0;
    print "Updating actual task complete counts...\n";
    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $dateValue = $resultSet->GetColumnValue(1);
       #print "   dateValue = $dateValue\n";
       my $actualFinishDBID = getDBID ($ACTUALFINISHDATE_TABLE,
                                         $ACTUALFINISHDATE_TABLE . "_name",
                                         $dateValue);
       my $numActualTasks = getAggregateValue($MEASURE,
                                              $ACTUAL_FINISH_DATE_FK,
                                              $actualFinishDBID,
                                              $dateValue);

       #printf "numActualTasks = $numActualTasks\n";

       updateAggregateTable($dateValue,
                            $MSP_AGGR_TASK_COMPLETE,
                            $numActualTasks);
       
    }
    printf "Updated $count rows with actual task complete.\n";
}


  
sub isOnTime{
    my ($actualFinishStr, $baselineFinishStr) = @_;

    if (length($baselineFinishStr) == 0) {
       # baseline finish date is not set so return True
       return BOOL_TRUE;
    } else {
       if (length($actualFinishStr) == 0) {
          # actual start is not set, so check if it's late.  If collectiondate is 
          # after baseline finish, then it's late
          if ($currentDate gt $baselineFinishStr) {
             return BOOL_FALSE;
          } else {
             return BOOL_TRUE;
          }
       } else {
          if ($actualFinishStr gt $baselineFinishStr) {
             return BOOL_FALSE;
          }
       }
    }
    return BOOL_TRUE;
}


# get date and set the global variables $currentDate and $currentTime
sub getCurrentDateTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    $mon = $mon + 1;
    $year = $year + 1900;

    if ($mday<10) {$mday="0$mday"; }  # add leading 0
    if ($mon<10) {$mon="0$mon"; }     # add leading 0

    $currentDate="$year-$mon-$mday";
    
    # set the global variable $currentTime
    $currentTime = $hour . ":" . $min . ":" . $sec;

}

# MS Project date format is something like "Fri 2/19/99";
# This will convert the date to "1999-02-19";
sub convertMSProjectDate {
    ($_) = @_;

    if ($_ == "NA") {
      return "";
    }
    # remove spaces and useless day
    #$_ = $temp;
    s/^\s+//;  # kill leading spaces
    s/^(?:Sun|Mon|Tue|Wed|Thu|Fri|Sat)[a-z]*,?\s*//i; # kill useless weekday
    
    #printf("converting ". $_ . "\n");
    @array = split(/\//, $_);
    my ($mon,$day,$yr) = @array;
    if ($yr < 100) {
      $yr += 1900;
    }
    my $newDate = sprintf("%04d-%02d-%02d", $yr, $mon, $day);  # add leading 0 on month and day

    
    return $newDate;
}

sub zeroPad {
    my ($n,$Str) = @_;

    #
    # Simple routine to add leading zeros to a string
    #
    while (length($Str) < $n) { $Str = "0".$Str; }
    return $Str;
}



sub logon {

    #
    # Use the internal DashboardSystem login to get access to the Dashboard database
    #
    my $session = Win32::OLE->new('CLEARQUEST.SESSION');
    my $session = CQPerlExt::CQSession_Build();  
    $session->UserLogon("dashboardsystem", "dashboardsystem", "tstd1", "Dashboard");
    return $session;
}


#
# Return the DBID of the specified field value from specified table.
#
sub getDBID {
    my ($tableName, $fieldName, $fieldValue) = @_;
    my $DBID = "0";

    #if (length($fieldValue) == 0) {
    #    return $DBID";
    #}

    #print getDBID started.\n";
    my $queryStr = "SELECT DBID FROM $tableName" .
                " WHERE $fieldName = '$fieldValue'";
    #print "getDBID(): Execute query $queryStr\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$DBID = $resultSet->GetColumnValue(1);
    }
    return $DBID;
}

#
# 
#
sub getAggregateValue {
    my (
        $measureField,
        $dateField,
        $dateDBID,
        $dateValue,
       ) = @_;


    my $queryStr = "SELECT SUM(". $measureField . ") FROM  " . $PROJECT_MEASURE_TABLE . 
                   " WHERE $dateField  > 0 AND $dateField <= $dateDBID AND " .
                   $COLLECTIONDATE_FK . " = (SELECT ". DBID . " FROM " .
                   $COLLECTIONDATE_TABLE . " WHERE " . 
                   $COLLECTIONDATE_TABLE . "_name = " . "'" . $currentDate . "')";
    #print "getAggregateValue(): Execute query: $queryStr\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    #print " query completed.\n";

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $result = $resultSet->GetColumnValue(1);
    }
    #print "    result = $result\n";
    return int($result);
}

#
# insert or update aggregate table
#
sub updateAggregateTable {
    my (
        $dateValue,
        $measureField,
        $measureValue,
       ) = @_;

    my $queryStr = "SELECT " . $PROJECT_AGGREGAGE_MEASURE_TABLE . "." . DBID .
                " FROM ". $PROJECT_AGGREGAGE_MEASURE_TABLE . ", " . $PROJECTDATE_TABLE . " WHERE " . 
                $PROJECT_AGGREGAGE_MEASURE_TABLE . "." . $MSP_AGGR_PROJECTDATE_FK . " = " . $PROJECTDATE_TABLE . "." . "dbid" .
                " AND " . $PROJECTDATE_TABLE . "." . $PROJECTDATE_TABLE . "_name = '$dateValue'";
    #print "updateAggregateTable(): Excute query: $queryStr\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    #print " query completed.\n";
    my $entity;

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       # do a MODIFY
       $DBID = $resultSet->GetColumnValue(1);

       #print "MODIFY aggregate record $DBID\n";
       $entity = $session->GetEntityByDbId($PROJECT_AGGREGAGE_MEASURE_TABLE, $DBID);
       $session->EditEntity($entity,"modify");
    }
    else {
       # do an INSERT
       #print "INSERT aggregate record into " . $PROJECT_AGGREGAGE_MEASURE_TABLE . "\n";
       $entity = $session->BuildEntity($PROJECT_AGGREGAGE_MEASURE_TABLE);
       #print "BuildEntity($tableName) succeeded\n";
 
    }
    #printf "Set " . $MSP_AGGR_PROJECTDATE_FK . " field to $dateValue\n";
    $entity->SetFieldValue($MSP_AGGR_PROJECTDATE_FK,$dateValue);
    
    #printf "SET $measureField  field to $measureValue\n";
    # convert measure to string
    my $measureValueStr = "$measureValue";
    $entity->SetFieldValue($measureField,$measureValueStr);

    #printf "Converted to string: $measureField = $measureValueStr\n";
    $retVal = $entity->Validate();
    #printf("entity->Validate() return value = $retVal\n");
    $entity->Commit();
    #printf("entity->Commit() return value = $retVal\n");

    #printf("Aggregate Measure Updated\n");
}


##################################################################################
#
# The following constants and subroutines are currently not used.  These are kept
# in this file as examples only.  If you wish, you can remove all of the below.
#

# Temp Fields
# Not Used! Keep as reference.
#   These fields hold temporary values that need to be transformed into actual values
use constant ACTUAL_COST_STR => "ActualCostStr";
use constant ACTUAL_DURATION_STR => "ActualDurationStr";
use constant ACTUAL_FINISH_DATE_STR => "actualfinish_str";
use constant ACTUAL_START_DATE_STR => "actualstart_str";
use constant ACTUAL_WORK_STR => "ActualWorkStr";
use constant ACWP_STR => "ACWPStr";
use constant BASELINE_COST_STR => "BaselineCostStr";
use constant BASELINE_DURATION_STR => "BaselineDurationStr";
use constant BASELINE_FINISH_DATE_STR => "baselinefinish_str";
use constant BASELINE_START_DATE_STR => "baselinestart_str";
use constant CURRENT_PLAN_START_DATE_STR => "curplanstart_str";
use constant CURRENT_PLAN_FINISH_DATE_STR => "curplanfinish_str";
use constant BASELINE_WORK_STR => "BaselineWorkStr";
use constant BCWP_STR => "BCWPStr";
use constant BCWS_STR => "BCWSStr";
use constant COST_STR => "CostStr";
use constant COST_VARIANCE_STR => "CostVarianceStr";
use constant FIXED_COST_STR => "FixedCostStr";
use constant PERCENT_COMPLETE_FK => "PercentCompleteStr";
use constant PERCENT_WORK_COMPLETE_FK => "PercentWorkCompleteStr";

# Not Used! Keep as reference.
sub buildTempProjTableResultSet {

    #
    # Return a result set containing the names of the time dimension tables using CQ API
    # WARNING: This doesn't work for some unknown reason!
    #
    
    print "buildTempProjTableResultSet started.\n";

    ####my $timeResultSet = $session->BuildSQLQuery("select name from actual where dbid <> 0 and table_type = '2' order by name");

    # build a querydef for the measure table
    my $queryDef = $session->BuildQuery($PROJECT_MEASURE_TABLE);
    $queryDef->BuildField(DBID);
    $queryDef->BuildField(ACTUAL_COST_STR);
    $queryDef->BuildField(ACTUAL_DURATION_STR);
    $queryDef->BuildField(ACTUAL_WORK_STR);
    $queryDef->BuildField(ACWP_STR);
    $queryDef->BuildField(BASELINE_COST_STR);
    $queryDef->BuildField(BASELINE_DURATION_STR);
    $queryDef->BuildField(BASELINE_WORK_STR);
    $queryDef->BuildField(BCWP_STR);
    $queryDef->BuildField(BCWS_STR);

    print "Build operator...\n";
    $filterOperator = $queryDef->BuildFilterOperator($CQPerlExt::CQ_BOOL_OP_AND);
    print "set filter: ", COLLECTIONDATE_FK, " == $currentDate\n";
    print "comp op eq == $CQPerlExt::CQ_COMP_OP_EQ\n";
    #$filterOperator->BuildFilter(COLLECTIONDATE_FK, $CQPerlExt::CQ_COMP_OP_EQ, $currentDate);
    $filterOperator->BuildFilter("taskid_fk", $CQPerlExt::CQ_COMP_OP_EQ, "2");

    print "BuildResultSet...\n";
    my $projResultSet = $session->BuildResultSet($queryDef);
    
    $projResultSet->Execute();

    print "buildTempProjTableResultSet completed.\n";
    return $projResultSet;
}

# 
# NOT USED - Save as example
#
sub getMinOrMaxDate {
    my ($dateTableName,
        $dateFkField,
        $minMax) = @_;
    my $dateValue;

    my $queryStr = "SELECT $minMax(". $dateTableName . "." . $dateTableName . "_name) " . 
                   " FROM  " . $PROJECT_MEASURE_TABLE . "," . $dateTableName .
                   " WHERE " . $PROJECT_MEASURE_TABLE . "." . $dateFkField . " > 0 AND " .
                   $PROJECT_MEASURE_TABLE . "." . $dateFkField . " = " . $dateTableName . "." . DBID;

    #print "getMinimumDate(): Execute query: $queryStr\n";
    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    my $count = 0;
    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $dateValue = $resultSet->GetColumnValue(1);
    }
    #printf "$minMax($dateFkField) = $dateValue\n";
    return $dateValue;

}

#
# Get minimum date
# 
# NOT USED - Save as example
#
sub getDate {
    my ($minMax) = @_;
    my $dateValue;

    my $queryStr = "SELECT $minMax(". $BASELINEFINISHDATE_TABLE . "." . $BASELINEFINISHDATE_TABLE . "_name) " . 
                   " FROM  " . $PROJECT_MEASURE_TABLE . "," . $BASELINEFINISHDATE_TABLE .
                   " WHERE " . $PROJECT_MEASURE_TABLE . "." . $BASELINE_FINISH_DATE_FK . " > 0 AND " .
                   $PROJECT_MEASURE_TABLE . "." . $BASELINE_FINISH_DATE_FK . " = " . $BASELINEFINISHDATE_TABLE . "." . DBID;

    #print "getMinimumDate(): Execute query: $queryStr\n";
    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    my $count = 0;
    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $dateValue = $resultSet->GetColumnValue(1);
    }
    #printf "baselineFinishMinDate = $dateValue\n";


    my $queryStr = "SELECT $minMax(". $CURPLANFINISHDATE_TABLE . "." . $CURPLANFINISHDATE_TABLE . "_name)" . 
                   " FROM  " . $PROJECT_MEASURE_TABLE . "," . $CURPLANFINISHDATE_TABLE .
                   " WHERE " . $PROJECT_MEASURE_TABLE . "." . $CURRENT_PLAN_FINISH_DATE_FK . " > 0 AND " .
                   $PROJECT_MEASURE_TABLE . "." . $CURRENT_PLAN_FINISH_DATE_FK . " = " . $CURPLANFINISHDATE_TABLE . "." . DBID;

    #print "getMinimumDate(): Execute query: $queryStr\n";
    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    my $count = 0;
    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $curPlanFinishDate = $resultSet->GetColumnValue(1);
    }
    #printf "curPlanFinishDate = $curPlanFinishDate\n";
    if ($minMax eq "MIN") {
       if ($curPlanFinishDate lt $dateValue) {
          $dateValue = $curPlanFinishDate;
       }
    } else {
       if ($curPlanFinishDate gt $dateValue) {
          $dateValue = $curPlanFinishDate;
       }
    }

    my $queryStr = "SELECT $minMax(". $ACTUALFINISHDATE_TABLE . "." . $ACTUALFINISHDATE_TABLE . "_name)" . 
                   " FROM  " . $PROJECT_MEASURE_TABLE . "," . $ACTUALFINISHDATE_TABLE .
                   " WHERE " . $PROJECT_MEASURE_TABLE . "." . $ACTUAL_FINISH_DATE_FK . " > 0 AND " .
                   $PROJECT_MEASURE_TABLE . "." . $ACTUAL_FINISH_DATE_FK . " = " . $ACTUALFINISHDATE_TABLE . "." . DBID;

    #print "getMinimumDate(): Execute query: $queryStr\n";
    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    my $count = 0;
    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
       $actualFinishDate = $resultSet->GetColumnValue(1);
    }
    #printf "actualFinishDate = $actualFinishDate\n";
    if ($minMax eq "MIN") {
       if ($actualFinishDate lt $dateValue) {
          $dateValue = $actualFinishDate;
       }
    } else {
       
    }

    #printf "dateValue = $dateValue\n";
    return $dateValue;
}

# NOT USED - keep for example
sub updateAggregateMeasuresNotUsed {

    my ($DBID,
        $currentDate,
        $baselineFinish,
        $currentPlanFinish,
        $actualFinish,
        $milestoneFk
        ) = @_;

    my $currentDateDBID = getDBID($COLLECTIONDATE_TABLE, $COLLECTIONDATE_TABLE . "_name", $currentDate);
    my $baselineFinishDBID = getDBID($BASELINEFINISHDATE_TABLE, $BASELINEFINISHDATE_TABLE . "_name", $baselineFinish);
    my $currentPlanFinishDBID = getDBID($CURPLANFINISHDATE_TABLE, $CURPLANFINISHDATE_TABLE . "_name", $currentPlanFinish);
    my $actualFinishDBID = getDBID($ACTUALFINISHDATE_TABLE, $ACTUALFINISHDATE_TABLE . "_name", $actualFinish);
    printf("updateAggregateMeasures():\n");
    printf("    DBID           = $DBID\n");
    printf("    currentDate    = $currentDateDBID, $currentDate\n");

    printf("    baselineFinish = $baselineFinishDBID, $baselineFinish\n");
    printf("    currentPlanFinish = $currentPlanFinishDBID, $currentPlanFinish\n");
    printf("    actualFinish      = $actualFinishDBID, $actualFinish\n");
    printf("    milestoneFk       = $milestoneFk\n");

    my $numBaselineTaskComplete = getAggregateValue(MSPTask_Measure,
                                                    $currentDate,
                                                    );

    #
    # Return a result set containing the names of the time dimension tables
    # using raw SQL command
    #

    #print updateAggregateMeasuresNotUsed started.\n";
    $queryStr = "SELECT " . DBID . "," .
                $MSP_AGGR_BP_TASK_COMPLETE . "," .
                $MSP_AGGR_CP_TASK_COMPLETE . "," .
                $MSP_AGGR_TASK_COMPLETE . "," .
                $MSP_AGGR_CALENDARDATE_FK . "," .
                $MSP_AGGR_MILESTONE_FK .
                " FROM " . $PROJECT_AGGREGAGE_MEASURE_TABLE .
                " WHERE " . $MSP_AGGR_COLLECTIONDATE_FK . " = $currentDateDBID";
    print "Excute query $queryStr...\n";

    my $projResultSet = $session->BuildSQLQuery($queryStr);
    $projResultSet->Execute();

    #print "buildTempProjTableResultSet completed.\n";
    #print "process eachrow...\n";
    my $count = 0;
    while ($projResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       $count = $count + 1;
    }
    printf ("Query returned $count rows in the aggregate table.\n");

    
}


#
# insert or update aggregate table
# NOT USED - Save as example
#
sub updateAggregateTableObsoleteSave {
    my (
        $dateValue,
        $numBPTasks,
        $numCPTasks,
        $numActualTasks
       ) = @_;

    my $queryStr = "SELECT " . $PROJECT_AGGREGAGE_MEASURE_TABLE . "." . DBID .
                " FROM ". $PROJECT_AGGREGAGE_MEASURE_TABLE . ", " . $PROJECTDATE_TABLE . " WHERE " . 
                $PROJECT_AGGREGAGE_MEASURE_TABLE . "." . $MSP_AGGR_PROJECTDATE_FK . " = " . $PROJECTDATE_TABLE . "." . "dbid" .
                " AND " . $PROJECTDATE_TABLE . "." . $PROJECTDATE_TABLE . "_name = '$dateValue'";
    #print "updateAggregateTable(): Excute query: $queryStr\n";

    my $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    #print " query completed.\n";
    my $entity;

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
       # do a MODIFY
       $DBID = $resultSet->GetColumnValue(1);

       #print "MODIFY aggregate record $DBID\n";
       $entity = $session->GetEntityByDbId($PROJECT_AGGREGAGE_MEASURE_TABLE, $DBID);
       $session->EditEntity($entity,"modify");
    }
    else {
       # do an INSERT
       #print "INSERT aggregate record into " . $PROJECT_AGGREGAGE_MEASURE_TABLE . "\n";
       $entity = $session->BuildEntity($PROJECT_AGGREGAGE_MEASURE_TABLE);
       #print "BuildEntity($tableName) succeeded\n";
 
    }
    #printf "Set " . $MSP_AGGR_PROJECTDATE_FK . " field to $dateValue\n";
    $entity->SetFieldValue($MSP_AGGR_PROJECTDATE_FK,$dateValue);
    
    # convert measure to string
    my $measuresValueStr = "$numBPTasks";
    $entity->SetFieldValue($MSP_AGGR_BP_TASK_COMPLETE,$measuresValueStr);
    $measuresValueStr = "$numCPTasks";
    $entity->SetFieldValue($MSP_AGGR_CP_TASK_COMPLETE,$measuresValueStr);
    $measuresValueStr = "$numActualTasks";
    $entity->SetFieldValue($MSP_AGGR_TASK_COMPLETE,$measuresValueStr);
    #printf "numBPTasks = $numBPTasks; numCPTasks = $numCPTasks; numActualTasks = $numActualTasks\n";
    $retVal = $entity->Validate();
    #printf("entity->Validate() return value = $retVal\n");
    $entity->Commit();
    #printf("entity->Commit() return value = $retVal\n");

    #printf("Aggregate Measure Updated\n");
}

